/**
 * Created by mgoria on 6/17/15.
 */

'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

var _get = function get(_x3, _x4, _x5) { var _again = true; _function: while (_again) { var object = _x3, property = _x4, receiver = _x5; desc = parent = getter = undefined; _again = false; if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { _x3 = parent; _x4 = property; _x5 = receiver; _again = true; continue _function; } } else if ('value' in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } } };

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

function _inherits(subClass, superClass) { if (typeof superClass !== 'function' && superClass !== null) { throw new TypeError('Super expression must either be null or a function, not ' + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var _deepKernel = require('deep-kernel');

var _deepKernel2 = _interopRequireDefault(_deepKernel);

var _ExceptionMissingLoginProviderException = require('./Exception/MissingLoginProviderException');

var _Token = require('./Token');

var _LocalToken = require('./LocalToken');

var _UserProvider = require('./UserProvider');

/**
 * Deep Security implementation
 */

var Security = (function (_Kernel$ContainerAware) {
  _inherits(Security, _Kernel$ContainerAware);

  /**
   * Defines all class private properties
   *
   * @param {String} identityPoolId
   * @param {Object} identityProviders
   */

  function Security() {
    var identityPoolId = arguments.length <= 0 || arguments[0] === undefined ? null : arguments[0];
    var identityProviders = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

    _classCallCheck(this, Security);

    _get(Object.getPrototypeOf(Security.prototype), 'constructor', this).call(this);

    this._identityPoolId = identityPoolId;
    this._identityProviders = identityProviders;
    this._token = null;
    this._userProvider = null;
    this._userProviderEndpoint = null;

    this._onTokenAvailable = [];
  }

  /**
   * @param {Function} callback
   * @returns {Security}
   */

  _createClass(Security, [{
    key: 'onTokenAvailable',
    value: function onTokenAvailable(callback) {
      this._onTokenAvailable.push(callback);

      return this;
    }

    /**
     * @returns {string}
     */
  }, {
    key: 'boot',

    /**
     * Booting a certain service
     *
     * @param {Kernel} kernel
     * @param {Function} callback
     */
    value: function boot(kernel, callback) {
      // @todo: remove this compatibility hook
      var globals = kernel.config.globals || kernel.config;

      this._identityProviders = globals.security.identityProviders;
      this._userProviderEndpoint = globals.userProviderEndpoint;

      this._identityPoolId = kernel.config.identityPoolId;

      callback();
    }

    /**
     * @param {String} name
     * @returns {Object}
     */
  }, {
    key: 'getLoginProviderConfig',
    value: function getLoginProviderConfig(name) {
      for (var providerName in this._identityProviders) {
        if (!this._identityProviders.hasOwnProperty(providerName)) {
          continue;
        }

        if (providerName === name) {
          return this._identityProviders[providerName];
        }
      }

      throw new _ExceptionMissingLoginProviderException.MissingLoginProviderException(name);
    }

    /**
     * @returns {Object}
     */
  }, {
    key: 'login',

    /**
     * @param {String} providerName
     * @param {String} userToken
     * @param {String} userId
     * @param {Function} callback
     * @returns {Token}
     */
    value: function login(providerName, userToken, userId, callback) {
      var _this = this;

      // check if providerName is defined
      this.getLoginProviderConfig(providerName);

      var TokenImplementation = this._localBackend ? _LocalToken.LocalToken : _Token.Token;

      this._token = new TokenImplementation(this._identityPoolId, providerName, userToken, userId);

      this._token.userProvider = this.userProvider;

      this._token.getCredentials(function () {
        for (var hookKey in _this._onTokenAvailable) {
          if (!_this._onTokenAvailable.hasOwnProperty(hookKey)) {
            continue;
          }

          _this._onTokenAvailable[hookKey](_this._token);
        }

        callback.apply(undefined, arguments);
      });

      return this._token;
    }

    /**
     * @param {Function} callback
     * @returns {Token}
     */
  }, {
    key: 'anonymousLogin',
    value: function anonymousLogin(callback) {
      var _this2 = this;

      var TokenImplementation = this._localBackend ? _LocalToken.LocalToken : _Token.Token;

      this._token = new TokenImplementation(this._identityPoolId);

      this._token.userProvider = this.userProvider;

      this._token.getCredentials(function () {
        for (var hookKey in _this2._onTokenAvailable) {
          if (!_this2._onTokenAvailable.hasOwnProperty(hookKey)) {
            continue;
          }

          _this2._onTokenAvailable[hookKey](_this2._token);
        }

        callback.apply(undefined, arguments);
      });

      return this._token;
    }
  }, {
    key: 'identityProviders',

    /**
     * @returns {Object}
     */
    get: function get() {
      return this._identityProviders;
    }

    /**
     * @returns {Object}
     */
  }, {
    key: 'identityPoolId',
    get: function get() {
      return this._identityPoolId;
    }

    /**
     * @returns {UserProvider}
     */
  }, {
    key: 'userProvider',
    get: function get() {
      if (!this._userProvider) {
        this._userProvider = new _UserProvider.UserProvider(this._userProviderEndpoint, this.container.get('resource'));
      }

      return this._userProvider;
    }
  }, {
    key: 'amazonLoginProviderConfig',
    get: function get() {
      return this.getLoginProviderConfig(Security.PROVIDER_AMAZON);
    }

    /**
     * @returns {Object}
     */
  }, {
    key: 'facebookLoginProviderConfig',
    get: function get() {
      return this.getLoginProviderConfig(Security.PROVIDER_FACEBOOK);
    }

    /**
     * @returns {Object}
     */
  }, {
    key: 'googleLoginProviderConfig',
    get: function get() {
      return this.getLoginProviderConfig(Security.PROVIDER_GOOGLE);
    }

    /**
     * @returns {null|Token}
     */
  }, {
    key: 'token',
    get: function get() {
      return this._token;
    }
  }], [{
    key: 'PROVIDER_AMAZON',
    get: function get() {
      return 'www.amazon.com';
    }

    /**
     * @returns {string}
     */
  }, {
    key: 'PROVIDER_FACEBOOK',
    get: function get() {
      return 'graph.facebook.com';
    }

    /**
     * @returns {string}
     */
  }, {
    key: 'PROVIDER_GOOGLE',
    get: function get() {
      return 'accounts.google.com';
    }
  }]);

  return Security;
})(_deepKernel2['default'].ContainerAware);

exports.Security = Security;