/**
 * Created by mgoria on 6/23/15.
 */

'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var _awsSdk = require('aws-sdk');

var _awsSdk2 = _interopRequireDefault(_awsSdk);

var _ExceptionAuthException = require('./Exception/AuthException');

/**
 * Security token holds details about logged user
 */

var Token = (function () {
  /**
   * @param {String} identityPoolId
   * @param {String} providerName
   * @param {String} providerUserToken
   * @param {String} providerUserId
   */

  function Token(identityPoolId) {
    var providerName = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];
    var providerUserToken = arguments.length <= 2 || arguments[2] === undefined ? null : arguments[2];
    var providerUserId = arguments.length <= 3 || arguments[3] === undefined ? null : arguments[3];

    _classCallCheck(this, Token);

    this._identityPoolId = identityPoolId;
    this._providerName = providerName;
    this._providerUserToken = providerUserToken;
    this._providerUserId = providerUserId;

    this._user = null;
    this._userProvider = null;
    this._identityId = null;
    this._credentials = null;

    this._isAnonymous = true;
  }

  /**
   * @param {Function} callback
   */

  _createClass(Token, [{
    key: 'getCredentials',
    value: function getCredentials() {
      var callback = arguments.length <= 0 || arguments[0] === undefined ? function () {
        return null;
      } : arguments[0];

      // @todo: set retries in a smarter way...
      _awsSdk2['default'].config.maxRetries = 3;

      var defaultRegion = _awsSdk2['default'].config.region;

      _awsSdk2['default'].config.update({
        region: Token._getRegionFromIdentityPoolId(this._identityPoolId)
      });

      var cognitoParams = {
        IdentityPoolId: this._identityPoolId
      };

      if (this._providerName && this._providerUserToken) {
        this._isAnonymous = false;
        cognitoParams.Logins = {};
        cognitoParams.Logins[this._providerName] = this._providerUserToken;
      }

      this._credentials = new _awsSdk2['default'].CognitoIdentityCredentials(cognitoParams);

      // update AWS credentials
      _awsSdk2['default'].config.credentials = this._credentials.refresh((function (error) {
        if (error) {
          callback(new _ExceptionAuthException.AuthException(error));
          return;
        }

        this._identityId = this._credentials.identityId;

        _awsSdk2['default'].config.update({
          accessKeyId: this._credentials.accessKeyId,
          secretAccessKey: this._credentials.secretAccessKey,
          sessionToken: this._credentials.sessionToken,
          region: defaultRegion });

        // restore to default region
        callback(null, this);
      }).bind(this));
    }

    /**
     * @param {String} identityPoolId
     * @returns {String}
     * @private
     */
  }, {
    key: 'getUser',

    /**
     * @param {Function} callback
     */
    value: function getUser(callback) {
      var _this = this;

      if (this.isAnonymous) {
        callback(null);
        return;
      }

      if (!this._user) {
        this._userProvider.loadUserByIdentityId(this.identityId, function (user) {
          if (user) {
            _this._user = user;
          }

          callback(user);
        });

        return;
      }

      callback(this._user);
    }
  }, {
    key: 'providerName',

    /**
     * @returns {String}
     */
    get: function get() {
      return this._providerName;
    }

    /**
     * @returns {String}
     */
  }, {
    key: 'providerUserToken',
    get: function get() {
      return this._providerUserToken;
    }

    /**
     * @returns {String}
     */
  }, {
    key: 'providerUserId',
    get: function get() {
      return this._providerUserId;
    }

    /**
     * @returns {String}
     */
  }, {
    key: 'identityId',
    get: function get() {
      return this._identityId;
    }

    /**
     * @returns {Object}
     */
  }, {
    key: 'credentials',
    get: function get() {
      return this._credentials;
    }

    /**
     * @returns {Boolean}
     */
  }, {
    key: 'isAnonymous',
    get: function get() {
      return this._isAnonymous;
    }

    /**
     * @param {UserProvider} userProvider
     */
  }, {
    key: 'userProvider',
    set: function set(userProvider) {
      this._userProvider = userProvider;
    }
  }], [{
    key: '_getRegionFromIdentityPoolId',
    value: function _getRegionFromIdentityPoolId(identityPoolId) {
      return identityPoolId.split(':')[0];
    }
  }]);

  return Token;
})();

exports.Token = Token;